@@ -7,12 +7,10 @@ use crate::parser::table::table;
 use crate::parser::trivia::{comment, line_ending, line_trailing, newline, ws};
 use crate::parser::value::value;
 use crate::parser::{TomlError, TomlParser};
-use crate::repr::Decor;
 use crate::table::TableKeyValue;
-use crate::Item;
-use combine::parser::char::char;
-use combine::parser::range::recognize;
-use combine::stream::position::Stream;
+use crate::{InternalString, Item};
+use combine::parser::byte::byte;
+use combine::stream::position::{IndexPositioner, Positioner, Stream};
 use combine::stream::RangeStream;
 use combine::Parser;
 use combine::*;
@@ -21,7 +19,11 @@ use std::mem;
 use std::ops::DerefMut;
 
 toml_parser!(parse_comment, parser, {
-    (comment(), line_ending()).map(|(c, e)| parser.borrow_mut().deref_mut().on_comment(c, e))
+    (comment(), line_ending()).and_then::<_, _, std::str::Utf8Error>(|(c, e)| {
+        let c = std::str::from_utf8(c)?;
+        parser.borrow_mut().deref_mut().on_comment(c, e);
+        Ok(())
+    })
 });
 
 toml_parser!(
@@ -31,7 +33,7 @@ toml_parser!(
 );
 
 toml_parser!(parse_newline, parser, {
-    recognize(newline()).map(|w| parser.borrow_mut().deref_mut().on_ws(w))
+    newline().map(|_| parser.borrow_mut().deref_mut().on_ws("\n"))
 });
 
 toml_parser!(keyval, parser, {
@@ -43,31 +45,33 @@ parser! {
     fn parse_keyval['a, I]()(I) -> (Vec<Key>, TableKeyValue)
     where
         [I: RangeStream<
-         Range = &'a str,
-         Token = char>,
-         I::Error: ParseError<char, &'a str, <I as StreamOnce>::Position>,
-         <I::Error as ParseError<char, &'a str, <I as StreamOnce>::Position>>::StreamError:
+         Range = &'a [u8],
+         Token = u8>,
+         I::Error: ParseError<u8, &'a [u8], <I as StreamOnce>::Position>,
+         <I::Error as ParseError<u8, &'a [u8], <I as StreamOnce>::Position>>::StreamError:
          From<std::num::ParseIntError> +
          From<std::num::ParseFloatError> +
+         From<std::str::Utf8Error> +
          From<crate::parser::errors::CustomError>
     ] {
         (
             key(),
-            char(KEYVAL_SEP),
+            byte(KEYVAL_SEP),
             (ws(), value(), line_trailing())
-        ).map(|(key, _, v)| {
-            let mut path = key.into_vec();
-            let key = path.pop().expect("Was vec1, so at least one exists");
+        ).and_then::<_, _, std::str::Utf8Error>(|(key, _, v)| {
+            let mut path = key;
+            let key = path.pop().expect("grammar ensures at least 1");
 
             let (pre, v, suf) = v;
+            let suf = std::str::from_utf8(suf)?;
             let v = v.decorated(pre, suf);
-            (
+            Ok((
                 path,
                 TableKeyValue {
                     key,
                     value: Item::Value(v),
                 }
-            )
+            ))
         })
     }
 }
@@ -81,7 +85,7 @@ impl TomlParser {
     //                ( ws keyval ws [ comment ] ) /
     //                ( ws table ws [ comment ] ) /
     //                  ws )
-    pub(crate) fn parse(s: &str) -> Result<Document, TomlError> {
+    pub(crate) fn parse(s: &[u8]) -> Result<Document, TomlError> {
         let mut parser = RefCell::new(Self::default());
         let input = Stream::new(s);
 
@@ -89,70 +93,84 @@ impl TomlParser {
             .with(choice((
                 eof(),
                 skip_many1(
-                    choice((
-                        parse_comment(&parser),
-                        keyval(&parser),
-                        table(&parser),
-                        parse_newline(&parser),
-                    ))
+                    look_ahead(any()).then(|e| {
+                        dispatch!(e;
+                            crate::parser::trivia::COMMENT_START_SYMBOL => parse_comment(&parser),
+                            crate::parser::table::STD_TABLE_OPEN => table(&parser),
+                            crate::parser::trivia::LF |
+                            crate::parser::trivia::CR => parse_newline(&parser),
+                            _ => keyval(&parser),
+                        )
+                    })
                     .skip(parse_ws(&parser)),
                 ),
             )))
             .easy_parse(input);
         match parsed {
-            Ok((_, ref rest)) if !rest.input.is_empty() => {
-                Err(TomlError::from_unparsed(rest.positioner, s))
-            }
+            Ok((_, ref rest)) if !rest.input.is_empty() => Err(TomlError::from_unparsed(
+                (&rest.positioner
+                    as &dyn Positioner<usize, Position = usize, Checkpoint = IndexPositioner>)
+                    .position(),
+                s,
+            )),
             Ok(..) => {
                 parser
                     .get_mut()
                     .finalize_table()
                     .map_err(|e| TomlError::custom(e.to_string()))?;
-                Ok(*parser.into_inner().document)
+                let trailing = parser.borrow().trailing.as_str().into();
+                parser.get_mut().document.trailing = trailing;
+                Ok(parser.into_inner().document)
             }
             Err(e) => Err(TomlError::new(e, s)),
         }
     }
 
     fn on_ws(&mut self, w: &str) {
-        self.document.trailing.push_str(w);
+        self.trailing.push_str(w);
     }
 
     fn on_comment(&mut self, c: &str, e: &str) {
-        self.document.trailing.push_str(c);
-        self.document.trailing.push_str(e);
+        self.trailing.push_str(c);
+        self.trailing.push_str(e);
     }
 
     fn on_keyval(&mut self, mut path: Vec<Key>, mut kv: TableKeyValue) -> Result<(), CustomError> {
         {
-            let prefix = mem::take(&mut self.document.trailing);
+            let prefix = mem::take(&mut self.trailing);
             let first_key = if path.is_empty() {
                 &mut kv.key
             } else {
                 &mut path[0]
             };
-            first_key.decor = Decor::new(
-                prefix + first_key.decor.prefix().unwrap_or_default(),
-                first_key.decor.suffix().unwrap_or_default(),
-            );
+            first_key
+                .decor
+                .set_prefix(prefix + first_key.decor.prefix().unwrap_or_default());
         }
 
         let table = &mut self.current_table;
-        let table = Self::descend_path(table, &path, 0, true)?;
+        let table = Self::descend_path(table, &path, true)?;
 
-        // "Since tables cannot be defined more than once, redefining such tables using a [table] header is not allowed"
-        let duplicate_key = table.contains_key(kv.key.get());
         // "Likewise, using dotted keys to redefine tables already defined in [table] form is not allowed"
         let mixed_table_types = table.is_dotted() == path.is_empty();
-        if duplicate_key || mixed_table_types {
-            Err(CustomError::DuplicateKey {
-                key: kv.key.into(),
+        if mixed_table_types {
+            return Err(CustomError::DuplicateKey {
+                key: kv.key.get().into(),
                 table: "<unknown>".into(), // TODO: get actual table name
-            })
-        } else {
-            let key = kv.key.get().to_owned();
-            table.items.insert(key, kv);
-            Ok(())
+            });
         }
+
+        let key: InternalString = kv.key.get_internal().into();
+        let old = table.items.insert(key.clone(), kv);
+        let duplicate_key = old.is_some();
+        // "Since tables cannot be defined more than once, redefining such tables using a [table] header is not allowed"
+        if duplicate_key {
+            return Err(CustomError::DuplicateKey {
+                key: key.as_str().into(),
+                table: "<unknown>".into(), // TODO: get actual table name
+            });
+        }
+
+        Ok(())
     }
 }
