@@ -25,10 +25,10 @@ use crate::key::Key;
 use crate::parser::errors::CustomError;
 use crate::repr::Decor;
 use crate::{ArrayOfTables, Document, Entry, Item, Table};
-use vec1::Vec1;
 
 pub(crate) struct TomlParser {
-    document: Box<Document>,
+    document: Document,
+    trailing: String,
     current_table_position: usize,
     current_table: Table,
     current_is_array: bool,
@@ -38,15 +38,16 @@ pub(crate) struct TomlParser {
 impl TomlParser {
     pub(crate) fn start_aray_table(
         &mut self,
-        path: Vec1<Key>,
+        path: Vec<Key>,
         decor: Decor,
     ) -> Result<(), CustomError> {
+        debug_assert!(!path.is_empty());
         debug_assert!(self.current_table.is_empty());
         debug_assert!(self.current_table_path.is_empty());
 
         // Look up the table on start to ensure the duplicate_key error points to the right line
         let root = self.document.as_table_mut();
-        let parent_table = Self::descend_path(root, &path[..path.len() - 1], 0, false)?;
+        let parent_table = Self::descend_path(root, &path[..path.len() - 1], false)?;
         let key = &path[path.len() - 1];
         let entry = parent_table
             .entry_format(key)
@@ -59,19 +60,20 @@ impl TomlParser {
         self.current_table.decor = decor;
         self.current_table.set_position(self.current_table_position);
         self.current_is_array = true;
-        self.current_table_path = path.into_vec();
+        self.current_table_path = path;
 
         Ok(())
     }
 
-    pub(crate) fn start_table(&mut self, path: Vec1<Key>, decor: Decor) -> Result<(), CustomError> {
+    pub(crate) fn start_table(&mut self, path: Vec<Key>, decor: Decor) -> Result<(), CustomError> {
+        debug_assert!(!path.is_empty());
         debug_assert!(self.current_table.is_empty());
         debug_assert!(self.current_table_path.is_empty());
 
         // 1. Look up the table on start to ensure the duplicate_key error points to the right line
         // 2. Ensure any child tables from an implicit table are preserved
         let root = self.document.as_table_mut();
-        let parent_table = Self::descend_path(root, &path[..path.len() - 1], 0, false)?;
+        let parent_table = Self::descend_path(root, &path[..path.len() - 1], false)?;
         let key = &path[path.len() - 1];
         if let Some(entry) = parent_table.remove(key.get()) {
             match entry {
@@ -86,7 +88,7 @@ impl TomlParser {
         self.current_table.decor = decor;
         self.current_table.set_position(self.current_table_position);
         self.current_is_array = false;
-        self.current_table_path = path.into_vec();
+        self.current_table_path = path;
 
         Ok(())
     }
@@ -100,7 +102,7 @@ impl TomlParser {
             assert!(root.is_empty());
             std::mem::swap(&mut table, root);
         } else if self.current_is_array {
-            let parent_table = Self::descend_path(root, &path[..path.len() - 1], 0, false)?;
+            let parent_table = Self::descend_path(root, &path[..path.len() - 1], false)?;
             let key = &path[path.len() - 1];
 
             let entry = parent_table
@@ -111,7 +113,7 @@ impl TomlParser {
                 .ok_or_else(|| duplicate_key(&path, path.len() - 1))?;
             array.push(table);
         } else {
-            let parent_table = Self::descend_path(root, &path[..path.len() - 1], 0, false)?;
+            let parent_table = Self::descend_path(root, &path[..path.len() - 1], false)?;
             let key = &path[path.len() - 1];
 
             let entry = parent_table.entry_format(key);
@@ -139,7 +141,8 @@ impl TomlParser {
 impl Default for TomlParser {
     fn default() -> Self {
         Self {
-            document: Box::new(Document::new()),
+            document: Document::new(),
+            trailing: String::new(),
             current_table_position: 0,
             current_table: Table::new(),
             current_is_array: false,
@@ -170,7 +173,7 @@ mod tests {
 
     macro_rules! parsed_eq {
         ($parsed:ident, $expected:expr) => {{
-            assert!($parsed.is_ok());
+            assert!($parsed.is_ok(), "{:?}", $parsed.err().unwrap());
             let (v, rest) = $parsed.unwrap();
             assert_eq!(v, $expected);
             assert!(rest.input.is_empty());
@@ -179,11 +182,11 @@ mod tests {
 
     macro_rules! parsed_float_eq {
         ($input:ident, $expected:expr) => {{
-            let parsed = numbers::float().easy_parse(Stream::new($input));
+            let parsed = numbers::float().easy_parse(Stream::new($input.as_bytes()));
             let (v, rest) = match parsed {
                 Ok(parsed) => parsed,
                 Err(err) => {
-                    panic!("Unexpected error for {:?}: {}", $input, err);
+                    panic!("Unexpected error for {:?}: {:?}", $input, err);
                 }
             };
             if $expected.is_nan() {
@@ -202,11 +205,11 @@ mod tests {
 
     macro_rules! parsed_value_eq {
         ($input:expr) => {
-            let parsed = value::value().easy_parse(Stream::new(*$input));
+            let parsed = value::value().easy_parse(Stream::new($input.as_bytes()));
             let (v, rest) = match parsed {
                 Ok(parsed) => parsed,
                 Err(err) => {
-                    panic!("Unexpected error for {:?}: {}", $input, err);
+                    panic!("Unexpected error for {:?}: {:?}", $input, err);
                 }
             };
             assert_eq!(v.to_string(), *$input);
@@ -216,7 +219,7 @@ mod tests {
 
     macro_rules! parsed_date_time_eq {
         ($input:expr, $is:ident) => {{
-            let parsed = value::value().easy_parse(Stream::new(*$input));
+            let parsed = value::value().easy_parse(Stream::new($input.as_bytes()));
             assert!(parsed.is_ok());
             let (v, rest) = parsed.unwrap();
             assert_eq!(v.to_string(), *$input);
@@ -242,12 +245,12 @@ mod tests {
             (&std::i64::MAX.to_string()[..], std::i64::MAX),
         ];
         for &(input, expected) in &cases {
-            let parsed = numbers::integer().easy_parse(Stream::new(input));
+            let parsed = numbers::integer().easy_parse(Stream::new(input.as_bytes()));
             parsed_eq!(parsed, expected);
         }
 
         let overflow = "1000000000000000000000000000000000";
-        let parsed = numbers::integer().easy_parse(Stream::new(overflow));
+        let parsed = numbers::integer().easy_parse(Stream::new(overflow.as_bytes()));
         assert!(parsed.is_err());
     }
 
@@ -281,7 +284,7 @@ mod tests {
     fn basic_string() {
         let input =
             r#""I'm a string. \"You can quote me\". Name\tJos\u00E9\nLocation\tSF. \U0002070E""#;
-        let parsed = strings::string().easy_parse(Stream::new(input));
+        let parsed = strings::string().easy_parse(Stream::new(input.as_bytes()));
         parsed_eq!(
             parsed,
             "I\'m a string. \"You can quote me\". Name\tJosé\nLocation\tSF. \u{2070E}"
@@ -303,14 +306,15 @@ Violets are blue"#,
         ];
 
         for &(input, expected) in &cases {
-            let parsed = strings::string().easy_parse(Stream::new(input));
+            dbg!(input);
+            let parsed = strings::string().easy_parse(Stream::new(input.as_bytes()));
             parsed_eq!(parsed, expected);
         }
 
         let invalid_cases = [r#""""  """#, r#""""  \""""#];
 
         for input in &invalid_cases {
-            let parsed = strings::ml_basic_string().easy_parse(Stream::new(*input));
+            let parsed = strings::ml_basic_string().easy_parse(Stream::new(input.as_bytes()));
             assert!(parsed.is_err());
         }
     }
@@ -331,7 +335,8 @@ The quick brown \
        """"#,
         ];
         for input in &inputs {
-            let parsed = strings::string().easy_parse(Stream::new(*input));
+            dbg!(input);
+            let parsed = strings::string().easy_parse(Stream::new(input.as_bytes()));
             parsed_eq!(parsed, "The quick brown fox jumps over the lazy dog.");
         }
         let empties = [
@@ -343,7 +348,7 @@ The quick brown \
 """"#,
         ];
         for empty in &empties {
-            let parsed = strings::string().easy_parse(Stream::new(*empty));
+            let parsed = strings::string().easy_parse(Stream::new(empty.as_bytes()));
             parsed_eq!(parsed, "");
         }
     }
@@ -358,7 +363,7 @@ The quick brown \
         ];
 
         for input in &inputs {
-            let parsed = strings::string().easy_parse(Stream::new(*input));
+            let parsed = strings::string().easy_parse(Stream::new(input.as_bytes()));
             parsed_eq!(parsed, &input[1..input.len() - 1]);
         }
     }
@@ -366,7 +371,7 @@ The quick brown \
     #[test]
     fn ml_literal_string() {
         let input = r#"'''I [dw]on't need \d{2} apples'''"#;
-        let parsed = strings::string().easy_parse(Stream::new(input));
+        let parsed = strings::string().easy_parse(Stream::new(input.as_bytes()));
         parsed_eq!(parsed, &input[3..input.len() - 3]);
         let input = r#"'''
 The first newline is
@@ -374,7 +379,7 @@ trimmed in raw strings.
    All other whitespace
    is preserved.
 '''"#;
-        let parsed = strings::string().easy_parse(Stream::new(input));
+        let parsed = strings::string().easy_parse(Stream::new(input.as_bytes()));
         parsed_eq!(parsed, &input[4..input.len() - 3]);
     }
 
@@ -437,11 +442,11 @@ trimmed in raw strings.
    "#,
         ];
         for input in &inputs {
-            let parsed = trivia::ws_comment_newline().easy_parse(Stream::new(*input));
+            let parsed = trivia::ws_comment_newline().easy_parse(Stream::new(input.as_bytes()));
             assert!(parsed.is_ok());
             let (t, rest) = parsed.unwrap();
             assert!(rest.input.is_empty());
-            assert_eq!(&t, input);
+            assert_eq!(t, input.as_bytes());
         }
     }
 
@@ -488,7 +493,7 @@ trimmed in raw strings.
 
         let invalid_inputs = [r#"["#, r#"[,]"#, r#"[,2]"#, r#"[1e165,,]"#];
         for input in &invalid_inputs {
-            let parsed = array::array().easy_parse(Stream::new(*input));
+            let parsed = array::array().easy_parse(Stream::new(input.as_bytes()));
             assert!(parsed.is_err());
         }
     }
@@ -507,7 +512,7 @@ trimmed in raw strings.
         }
         let invalid_inputs = [r#"{a = 1e165"#, r#"{ hello = "world", a = 2, hello = 1}"#];
         for input in &invalid_inputs {
-            let parsed = inline_table::inline_table().easy_parse(Stream::new(*input));
+            let parsed = inline_table::inline_table().easy_parse(Stream::new(input.as_bytes()));
             assert!(parsed.is_err());
         }
     }
@@ -521,10 +526,10 @@ trimmed in raw strings.
         ];
 
         for &(input, expected) in &cases {
-            let parsed = key::simple_key().easy_parse(Stream::new(input));
+            let parsed = key::simple_key().easy_parse(Stream::new(input.as_bytes()));
             assert!(parsed.is_ok());
             let ((.., k), rest) = parsed.unwrap();
-            assert_eq!(k, expected);
+            assert_eq!(k.as_str(), expected);
             assert_eq!(rest.input.len(), 0);
         }
     }
@@ -611,7 +616,7 @@ key = "value"
 "#,
         ];
         for document in &documents {
-            let doc = TomlParser::parse(document);
+            let doc = TomlParser::parse(document.as_bytes());
             let doc = match doc {
                 Ok(doc) => doc,
                 Err(err) => {
@@ -630,7 +635,7 @@ key = "value"
         let invalid_inputs = [r#" hello = 'darkness' # my old friend
 $"#];
         for document in &invalid_inputs {
-            let doc = TomlParser::parse(document);
+            let doc = TomlParser::parse(document.as_bytes());
 
             assert!(doc.is_err());
         }
