@@ -5,7 +5,7 @@ use crate::parser::trivia::line_trailing;
 use crate::parser::TomlParser;
 use crate::repr::Decor;
 use crate::{Item, Table};
-use combine::parser::char::char;
+use combine::parser::byte::byte;
 use combine::parser::range::range;
 use combine::stream::RangeStream;
 use combine::*;
@@ -15,24 +15,23 @@ use std::mem;
 // https://github.com/rust-lang/rust/issues/41358
 #[allow(unused_imports)]
 use std::ops::DerefMut;
-use vec1::Vec1;
 
 // std-table-open  = %x5B ws     ; [ Left square bracket
-const STD_TABLE_OPEN: char = '[';
+pub(crate) const STD_TABLE_OPEN: u8 = b'[';
 // std-table-close = ws %x5D     ; ] Right square bracket
-const STD_TABLE_CLOSE: char = ']';
+const STD_TABLE_CLOSE: u8 = b']';
 // array-table-open  = %x5B.5B ws  ; [[ Double left square bracket
-const ARRAY_TABLE_OPEN: &str = "[[";
+const ARRAY_TABLE_OPEN: &[u8] = b"[[";
 // array-table-close = ws %x5D.5D  ; ]] Double right quare bracket
-const ARRAY_TABLE_CLOSE: &str = "]]";
+const ARRAY_TABLE_CLOSE: &[u8] = b"]]";
 
 // ;; Standard Table
 
 // std-table = std-table-open key *( table-key-sep key) std-table-close
 toml_parser!(std_table, parser, {
     (
-        between(char(STD_TABLE_OPEN), char(STD_TABLE_CLOSE), key()),
-        line_trailing(),
+        between(byte(STD_TABLE_OPEN), byte(STD_TABLE_CLOSE), key()),
+        line_trailing().and_then(|t| std::str::from_utf8(t)),
     )
         .and_then(|(h, t)| parser.borrow_mut().deref_mut().on_std_header(h, t))
 });
@@ -43,7 +42,7 @@ toml_parser!(std_table, parser, {
 toml_parser!(array_table, parser, {
     (
         between(range(ARRAY_TABLE_OPEN), range(ARRAY_TABLE_CLOSE), key()),
-        line_trailing(),
+        line_trailing().and_then(|t| std::str::from_utf8(t)),
     )
         .and_then(|(h, t)| parser.borrow_mut().deref_mut().on_array_header(h, t))
 });
@@ -55,12 +54,13 @@ parser! {
     pub(crate) fn table['a, 'b, I](parser: &'b RefCell<TomlParser>)(I) -> ()
     where
         [I: RangeStream<
-         Range = &'a str,
-         Token = char>,
-         I::Error: ParseError<char, &'a str, <I as StreamOnce>::Position>,
-         <I::Error as ParseError<char, &'a str, <I as StreamOnce>::Position>>::StreamError:
+         Range = &'a [u8],
+         Token = u8>,
+         I::Error: ParseError<u8, &'a [u8], <I as StreamOnce>::Position>,
+         <I::Error as ParseError<u8, &'a [u8], <I as StreamOnce>::Position>>::StreamError:
          From<std::num::ParseIntError> +
          From<std::num::ParseFloatError> +
+         From<std::str::Utf8Error> +
          From<crate::parser::errors::CustomError>
     ]    {
         array_table(parser)
@@ -82,13 +82,12 @@ pub(crate) fn duplicate_key(path: &[Key], i: usize) -> CustomError {
 }
 
 impl TomlParser {
-    pub(crate) fn descend_path<'a>(
-        table: &'a mut Table,
-        path: &'a [Key],
-        i: usize,
+    pub(crate) fn descend_path<'t, 'k>(
+        mut table: &'t mut Table,
+        path: &'k [Key],
         dotted: bool,
-    ) -> Result<&'a mut Table, CustomError> {
-        if let Some(key) = path.get(i) {
+    ) -> Result<&'t mut Table, CustomError> {
+        for (i, key) in path.iter().enumerate() {
             let entry = table.entry_format(key).or_insert_with(|| {
                 let mut new_table = Table::new();
                 new_table.set_implicit(true);
@@ -97,40 +96,41 @@ impl TomlParser {
                 Item::Table(new_table)
             });
             match *entry {
-                Item::Value(..) => Err(duplicate_key(path, i)),
+                Item::Value(..) => {
+                    return Err(duplicate_key(path, i));
+                }
                 Item::ArrayOfTables(ref mut array) => {
                     debug_assert!(!array.is_empty());
 
                     let index = array.len() - 1;
                     let last_child = array.get_mut(index).unwrap();
 
-                    Self::descend_path(last_child, path, i + 1, dotted)
+                    table = last_child;
                 }
                 Item::Table(ref mut sweet_child_of_mine) => {
-                    TomlParser::descend_path(sweet_child_of_mine, path, i + 1, dotted)
+                    table = sweet_child_of_mine;
                 }
                 _ => unreachable!(),
             }
-        } else {
-            Ok(table)
         }
+        Ok(table)
     }
 
-    fn on_std_header(&mut self, path: Vec1<Key>, trailing: &str) -> Result<(), CustomError> {
+    fn on_std_header(&mut self, path: Vec<Key>, trailing: &str) -> Result<(), CustomError> {
         debug_assert!(!path.is_empty());
 
         self.finalize_table()?;
-        let leading = mem::take(&mut self.document.trailing);
+        let leading = mem::take(&mut self.trailing);
         self.start_table(path, Decor::new(leading, trailing))?;
 
         Ok(())
     }
 
-    fn on_array_header(&mut self, path: Vec1<Key>, trailing: &str) -> Result<(), CustomError> {
+    fn on_array_header(&mut self, path: Vec<Key>, trailing: &str) -> Result<(), CustomError> {
         debug_assert!(!path.is_empty());
 
         self.finalize_table()?;
-        let leading = mem::take(&mut self.document.trailing);
+        let leading = mem::take(&mut self.trailing);
         self.start_aray_table(path, Decor::new(leading, trailing))?;
 
         Ok(())
