@@ -55,11 +55,11 @@ impl Table {
     pub fn get_values(&self) -> Vec<(Vec<&Key>, &Value)> {
         let mut values = Vec::new();
         let root = Vec::new();
-        self.get_values_internal(&root, &mut values);
+        self.append_values(&root, &mut values);
         values
     }
 
-    fn get_values_internal<'s, 'c>(
+    fn append_values<'s, 'c>(
         &'s self,
         parent: &[&'s Key],
         values: &'c mut Vec<(Vec<&'s Key>, &'s Value)>,
@@ -69,10 +69,18 @@ impl Table {
             path.push(&value.key);
             match &value.value {
                 Item::Table(table) if table.is_dotted() => {
-                    table.get_values_internal(&path, values);
+                    table.append_values(&path, values);
                 }
                 Item::Value(value) => {
-                    values.push((path, value));
+                    if let Some(table) = value.as_inline_table() {
+                        if table.is_dotted() {
+                            table.append_values(&path, values);
+                        } else {
+                            values.push((path, value));
+                        }
+                    } else {
+                        values.push((path, value));
+                    }
                 }
                 _ => {}
             }
@@ -388,7 +396,7 @@ pub type Iter<'a> = Box<dyn Iterator<Item = (&'a str, &'a Item)> + 'a>;
 pub type IterMut<'a> = Box<dyn Iterator<Item = (&'a str, &'a mut Item)> + 'a>;
 
 /// This trait represents either a `Table`, or an `InlineTable`.
-pub trait TableLike {
+pub trait TableLike: crate::private::Sealed {
     /// Returns an iterator over key/value pairs.
     fn iter(&self) -> Iter<'_>;
     /// Returns an mutable iterator over all key/value pairs, including empty.
@@ -423,6 +431,10 @@ pub trait TableLike {
     ///
     /// Doesn't affect subtables or subarrays.
     fn sort_values(&mut self);
+    /// Change this table's dotted status
+    fn set_dotted(&mut self, yes: bool);
+    /// Check if this is a wrapper for dotted keys, rather than a standard table
+    fn is_dotted(&self) -> bool;
 
     /// Returns the decor associated with a given key of the table.
     fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor>;
@@ -462,6 +474,13 @@ impl TableLike for Table {
     fn sort_values(&mut self) {
         self.sort_values()
     }
+    fn is_dotted(&self) -> bool {
+        self.is_dotted()
+    }
+    fn set_dotted(&mut self, yes: bool) {
+        self.set_dotted(yes)
+    }
+
     fn key_decor_mut(&mut self, key: &str) -> Option<&mut Decor> {
         self.key_decor_mut(key)
     }
