@@ -1,3 +1,8 @@
+use pretty_assertions::assert_eq;
+use std::fmt;
+use std::iter::FromIterator;
+use toml_edit::{array, table, value, Document, Item, Key, Table, Value};
+
 macro_rules! parse_key {
     ($s:expr) => {{
         let key = $s.parse::<Key>();
@@ -13,76 +18,68 @@ macro_rules! as_table {
     }};
 }
 
-// rusfmt, U Can't Touch This
-#[cfg(test)]
-#[rustfmt::skip]
-mod tests {
-    use toml_edit::{Document, Key, Value, Table, Item, value, table, array};
-    use std::iter::FromIterator;
-    use std::fmt;
-    use pretty_assertions::assert_eq;
-
-    // Copied from https://github.com/colin-kiegel/rust-pretty-assertions/issues/24
-    /// Wrapper around string slice that makes debug output `{:?}` to print string same way as `{}`.
-    /// Used in different `assert*!` macros in combination with `pretty_assertions` crate to make
-    /// test failures to show nice diffs.
-    #[derive(PartialEq, Eq)]
-    struct PrettyString<'a>(pub &'a str);
-    /// Make diff to display string as multi-line string
-    impl<'a> fmt::Debug for PrettyString<'a> {
-        fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-            f.write_str(self.0)
-        }
+// Copied from https://github.com/colin-kiegel/rust-pretty-assertions/issues/24
+/// Wrapper around string slice that makes debug output `{:?}` to print string same way as `{}`.
+/// Used in different `assert*!` macros in combination with `pretty_assertions` crate to make
+/// test failures to show nice diffs.
+#[derive(PartialEq, Eq)]
+struct PrettyString<'a>(pub &'a str);
+/// Make diff to display string as multi-line string
+impl<'a> fmt::Debug for PrettyString<'a> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.write_str(self.0)
     }
+}
 
-    struct Test {
-        doc: Document,
-    }
+struct Test {
+    doc: Document,
+}
 
-    fn given(input: &str) -> Test {
-        let doc = input.parse::<Document>();
-        assert!(doc.is_ok());
-        Test {
-            doc: doc.unwrap(),
-        }
-    }
+fn given(input: &str) -> Test {
+    let doc = input.parse::<Document>();
+    assert!(doc.is_ok());
+    Test { doc: doc.unwrap() }
+}
 
-    impl Test {
-        fn running<F>(&mut self, func: F) -> &mut Self
-            where F: Fn(&mut Table)
+impl Test {
+    fn running<F>(&mut self, func: F) -> &mut Self
+    where
+        F: Fn(&mut Table),
+    {
         {
-            {
-                let root = self.doc.as_table_mut();
-                func(root);
-            }
-            self
-        }
-
-        fn produces_display(&self, expected: &str) -> &Self {
-            assert_eq!(
-                PrettyString(expected),
-                PrettyString(&self.doc.to_string()));
-            self
+            let root = self.doc.as_table_mut();
+            func(root);
         }
+        self
     }
 
+    fn produces_display(&self, expected: &str) -> &Self {
+        assert_eq!(PrettyString(expected), PrettyString(&self.doc.to_string()));
+        self
+    }
+}
+
 // insertion
 
 #[test]
 fn test_insert_leaf_table() {
-    given(r#"
+    given(
+        r#"
         [servers]
 
         [servers.alpha]
         ip = "10.0.0.1"
         dc = "eqdc10"
 
-        [other.table]"#
-    ).running(|root| {
+        [other.table]"#,
+    )
+    .running(|root| {
         root["servers"]["beta"] = table();
         root["servers"]["beta"]["ip"] = value("10.0.0.2");
         root["servers"]["beta"]["dc"] = value("eqdc10");
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         [servers]
 
         [servers.alpha]
@@ -94,21 +91,25 @@ ip = "10.0.0.2"
 dc = "eqdc10"
 
         [other.table]
-"#
+"#,
     );
 }
 
 #[test]
 fn test_inserted_leaf_table_goes_after_last_sibling() {
-    given(r#"
+    given(
+        r#"
         [package]
         [dependencies]
         [[example]]
         [dependencies.opencl]
-        [dev-dependencies]"#
-    ).running(|root| {
+        [dev-dependencies]"#,
+    )
+    .running(|root| {
         root["dependencies"]["newthing"] = table();
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         [package]
         [dependencies]
         [[example]]
@@ -116,30 +117,33 @@ fn test_inserted_leaf_table_goes_after_last_sibling() {
 
 [dependencies.newthing]
         [dev-dependencies]
-"#);
+"#,
+    );
 }
 
 #[test]
 fn test_inserting_tables_from_different_parsed_docs() {
-    given(
-        "[a]"
-    ).running(|root| {
-        let other = "[b]".parse::<Document>().unwrap();
-        root["b"] = other["b"].clone();
-    }).produces_display(
-        "[a]\n[b]\n"
-    );
+    given("[a]")
+        .running(|root| {
+            let other = "[b]".parse::<Document>().unwrap();
+            root["b"] = other["b"].clone();
+        })
+        .produces_display("[a]\n[b]\n");
 }
 #[test]
 fn test_insert_nonleaf_table() {
-    given(r#"
-        [other.table]"#
-    ).running(|root| {
+    given(
+        r#"
+        [other.table]"#,
+    )
+    .running(|root| {
         root["servers"] = table();
         root["servers"]["alpha"] = table();
         root["servers"]["alpha"]["ip"] = value("10.0.0.1");
         root["servers"]["alpha"]["dc"] = value("eqdc10");
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         [other.table]
 
 [servers]
@@ -147,16 +151,18 @@ fn test_insert_nonleaf_table() {
 [servers.alpha]
 ip = "10.0.0.1"
 dc = "eqdc10"
-"#
+"#,
     );
 }
 
 #[test]
 fn test_insert_array() {
-    given(r#"
+    given(
+        r#"
         [package]
-        title = "withoutarray""#
-    ).running(|root| {
+        title = "withoutarray""#,
+    )
+    .running(|root| {
         root["bin"] = array();
         assert!(root["bin"].is_array_of_tables());
         let array = root["bin"].as_array_of_tables_mut().unwrap();
@@ -166,7 +172,9 @@ fn test_insert_array() {
             array.push(table);
         }
         array.push(Table::new());
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         [package]
         title = "withoutarray"
 
@@ -174,27 +182,30 @@ fn test_insert_array() {
 hello = "world"
 
 [[bin]]
-"#
+"#,
     );
 }
 
-
 #[test]
 fn test_insert_values() {
-    given(r#"
-        [tbl.son]"#
-    ).running(|root| {
+    given(
+        r#"
+        [tbl.son]"#,
+    )
+    .running(|root| {
         root["tbl"]["key1"] = value("value1");
         root["tbl"]["key2"] = value(42);
         root["tbl"]["key3"] = value(8.1415926);
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
 [tbl]
 key1 = "value1"
 key2 = 42
 key3 = 8.1415926
 
         [tbl.son]
-"#
+"#,
     );
 }
 
@@ -202,7 +213,8 @@ key3 = 8.1415926
 
 #[test]
 fn test_remove_leaf_table() {
-    given(r#"
+    given(
+        r#"
         [servers]
 
         # Indentation (tabs and/or spaces) is allowed but not required
@@ -212,25 +224,28 @@ fn test_remove_leaf_table() {
 
         [servers.beta]
         ip = "10.0.0.2"
-        dc = "eqdc10""#
-    ).running(|root| {
+        dc = "eqdc10""#,
+    )
+    .running(|root| {
         let servers = root.get_mut("servers").unwrap();
         let servers = as_table!(servers);
         assert!(servers.remove("alpha").is_some());
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         [servers]
 
         [servers.beta]
         ip = "10.0.0.2"
         dc = "eqdc10"
-"#
+"#,
     );
 }
 
-
 #[test]
 fn test_remove_nonleaf_table() {
-    given(r#"
+    given(
+        r#"
         title = "not relevant"
 
         # comment 1
@@ -262,9 +277,13 @@ fn test_remove_nonleaf_table() {
         key41 = 41 # comment 4.3
 
 
-    "#).running(|root| {
+    "#,
+    )
+    .running(|root| {
         assert!(root.remove("a").is_some());
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         title = "not relevant"
         # comment 2
         [b] # comment 2.1
@@ -274,12 +293,14 @@ fn test_remove_nonleaf_table() {
            [some.other.table]
 
 
-    "#);
+    "#,
+    );
 }
 
 #[test]
 fn test_remove_array_entry() {
-    given(r#"
+    given(
+        r#"
         [package]
         name = "hello"
         version = "1.0.0"
@@ -293,15 +314,18 @@ fn test_remove_array_entry() {
 
         [[bin]]
         name = "delete me please"
-        path = "src/bin/dmp/main.rs""#
-    ).running(|root| {
+        path = "src/bin/dmp/main.rs""#,
+    )
+    .running(|root| {
         let dmp = root.get_mut("bin").unwrap();
         assert!(dmp.is_array_of_tables());
         let dmp = dmp.as_array_of_tables_mut().unwrap();
         assert_eq!(dmp.len(), 2);
         dmp.remove(1);
         assert_eq!(dmp.len(), 1);
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         [package]
         name = "hello"
         version = "1.0.0"
@@ -312,13 +336,14 @@ fn test_remove_array_entry() {
 
         [dependencies]
         nom = "4.0" # future is here
-"#
+"#,
     );
 }
 
 #[test]
 fn test_remove_array() {
-    given(r#"
+    given(
+        r#"
         [package]
         name = "hello"
         version = "1.0.0"
@@ -332,29 +357,33 @@ fn test_remove_array() {
 
         [[bin]]
         name = "delete me please"
-        path = "src/bin/dmp/main.rs""#
-    ).running(|root| {
+        path = "src/bin/dmp/main.rs""#,
+    )
+    .running(|root| {
         assert!(root.remove("bin").is_some());
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         [package]
         name = "hello"
         version = "1.0.0"
 
         [dependencies]
         nom = "4.0" # future is here
-"#
+"#,
     );
 }
 
-
 #[test]
 fn test_remove_value() {
-    given(r#"
+    given(
+        r#"
         name = "hello"
         # delete this
         version = "1.0.0" # please
-        documentation = "https://docs.rs/hello""#
-    ).running(|root| {
+        documentation = "https://docs.rs/hello""#,
+    )
+    .running(|root| {
         let value = root.remove("version");
         assert!(value.is_some());
         let value = value.unwrap();
@@ -363,19 +392,23 @@ fn test_remove_value() {
         assert!(value.is_str());
         let value = value.as_str().unwrap();
         assert_eq!(value, "1.0.0");
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         name = "hello"
         documentation = "https://docs.rs/hello"
-"#
+"#,
     );
 }
 
 #[test]
 fn test_remove_last_value_from_implicit() {
-    given(r#"
+    given(
+        r#"
         [a]
-        b = 1"#
-    ).running(|root| {
+        b = 1"#,
+    )
+    .running(|root| {
         let a = root.get_mut("a").unwrap();
         assert!(a.is_table());
         let a = as_table!(a);
@@ -386,14 +419,16 @@ fn test_remove_last_value_from_implicit() {
         assert!(value.is_value());
         let value = value.as_value().unwrap();
         assert_eq!(value.as_integer(), Some(1));
-    }).produces_display(r#""#);
+    })
+    .produces_display(r#""#);
 }
 
 // values
 
 #[test]
 fn test_sort_values() {
-    given(r#"
+    given(
+        r#"
         [a.z]
 
         [a]
@@ -402,12 +437,15 @@ fn test_sort_values() {
         a = 1
         c = 3
 
-        [a.y]"#
-    ).running(|root| {
+        [a.y]"#,
+    )
+    .running(|root| {
         let a = root.get_mut("a").unwrap();
         let a = as_table!(a);
         a.sort_values();
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         [a.z]
 
         [a]
@@ -417,17 +455,20 @@ fn test_sort_values() {
         c = 3
 
         [a.y]
-"#);
+"#,
+    );
 }
 
 #[test]
 fn test_set_position() {
-    given(r#"
+    given(
+        r#"
         [package]
         [dependencies]
         [dependencies.opencl]
-        [dev-dependencies]"#
-    ).running(|root| {
+        [dev-dependencies]"#,
+    )
+    .running(|root| {
         for (header, table) in root.iter_mut() {
             if header == "dependencies" {
                 let tab = as_table!(table);
@@ -436,73 +477,86 @@ fn test_set_position() {
                 as_table!(segmented).set_position(5)
             }
         }
-    }).produces_display(r#"        [dependencies]
+    })
+    .produces_display(
+        r#"        [dependencies]
 
         [package]
         [dev-dependencies]
         [dependencies.opencl]
-"#);
-}
-
-#[test]
-fn test_multiple_zero_positions() {
-    given(r#"
-        [package]
-        [dependencies]
-        [dependencies.opencl]
-        a=""
-        [dev-dependencies]"#
-    ).running(|root| {
-        for (_, table) in root.iter_mut() {
-            as_table!(table).set_position(0)
-        }
-    }).produces_display(r#"
-        [package]
-        [dependencies]
-        [dev-dependencies]
-        [dependencies.opencl]
-        a=""
-"#);
-}
-
-#[test]
-fn test_multiple_max_usize_positions() {
-    given(r#"
-        [package]
-        [dependencies]
-        [dependencies.opencl]
-        a=""
-        [dev-dependencies]"#
-    ).running(|root| {
-        for (_, table) in root.iter_mut() {
-            as_table!(table).set_position(usize::MAX)
-        }
-    }).produces_display(r#"        [dependencies.opencl]
-        a=""
-
-        [package]
-        [dependencies]
-        [dev-dependencies]
-"#);
-}
-
-macro_rules! as_array {
-    ($entry:ident) => (
-        {
-            assert!($entry.is_value());
-            let a = $entry.as_value_mut().unwrap();
-            assert!(a.is_array());
-            a.as_array_mut().unwrap()
-        }
+"#,
     );
 }
 
+#[test]
+fn test_multiple_zero_positions() {
+    given(
+        r#"
+        [package]
+        [dependencies]
+        [dependencies.opencl]
+        a=""
+        [dev-dependencies]"#,
+    )
+    .running(|root| {
+        for (_, table) in root.iter_mut() {
+            as_table!(table).set_position(0)
+        }
+    })
+    .produces_display(
+        r#"
+        [package]
+        [dependencies]
+        [dev-dependencies]
+        [dependencies.opencl]
+        a=""
+"#,
+    );
+}
+
+#[test]
+fn test_multiple_max_usize_positions() {
+    given(
+        r#"
+        [package]
+        [dependencies]
+        [dependencies.opencl]
+        a=""
+        [dev-dependencies]"#,
+    )
+    .running(|root| {
+        for (_, table) in root.iter_mut() {
+            as_table!(table).set_position(usize::MAX)
+        }
+    })
+    .produces_display(
+        r#"        [dependencies.opencl]
+        a=""
+
+        [package]
+        [dependencies]
+        [dev-dependencies]
+"#,
+    );
+}
+
+macro_rules! as_array {
+    ($entry:ident) => {{
+        assert!($entry.is_value());
+        let a = $entry.as_value_mut().unwrap();
+        assert!(a.is_array());
+        a.as_array_mut().unwrap()
+    }};
+}
+
 #[test]
 fn test_insert_replace_into_array() {
-    given(r#"
+    given(
+        r#"
         a = [1,2,3]
-        b = []"#
-    ).running(|root| {
+        b = []"#,
+    )
+    .running(|root| {
         {
             let a = root.get_mut("a").unwrap();
             let a = as_array!(a);
@@ -527,24 +581,31 @@ fn test_insert_replace_into_array() {
         // This should preserve formatting.
         assert_eq!(b.replace(2, "zoink").as_str(), Some("boop"));
         // This should replace formatting.
-        assert_eq!(b.replace_formatted(4, Value::from("yikes").decorated("  ", "")).as_str(), Some("test"));
+        assert_eq!(
+            b.replace_formatted(4, Value::from("yikes").decorated("  ", ""))
+                .as_str(),
+            Some("test")
+        );
         dbg!(root);
-
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         a = [1, 2, 3, 4]
         b = ["hello", "beep",   "zoink"   ,
 "world"
 ,  "yikes"]
-"#
+"#,
     );
 }
 
 #[test]
 fn test_remove_from_array() {
-    given(r#"
+    given(
+        r#"
         a = [1, 2, 3, 4]
-        b = ["hello"]"#
-    ).running(|root| {
+        b = ["hello"]"#,
+    )
+    .running(|root| {
         {
             let a = root.get_mut("a").unwrap();
             let a = as_array!(a);
@@ -557,30 +618,32 @@ fn test_remove_from_array() {
         assert_eq!(b.len(), 1);
         assert!(b.remove(0).is_str());
         assert!(b.is_empty());
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         a = [1, 2, 3]
         b = []
-"#
+"#,
     );
 }
 
 macro_rules! as_inline_table {
-    ($entry:ident) => (
-        {
-            assert!($entry.is_value());
-            let a = $entry.as_value_mut().unwrap();
-            assert!(a.is_inline_table());
-            a.as_inline_table_mut().unwrap()
-        }
-    );
+    ($entry:ident) => {{
+        assert!($entry.is_value());
+        let a = $entry.as_value_mut().unwrap();
+        assert!(a.is_inline_table());
+        a.as_inline_table_mut().unwrap()
+    }};
 }
 
 #[test]
 fn test_insert_into_inline_table() {
-    given(r#"
+    given(
+        r#"
         a = {a=2,  c = 3}
-        b = {}"#
-    ).running(|root| {
+        b = {}"#,
+    )
+    .running(|root| {
         {
             let a = root.get_mut("a").unwrap();
             let a = as_inline_table!(a);
@@ -596,19 +659,23 @@ fn test_insert_into_inline_table() {
         b.get_or_insert("hello", "world");
         assert_eq!(b.len(), 1);
         b.fmt()
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         a = { a = 2, c = 3, b = 42 }
         b = { hello = "world" }
-"#
+"#,
     );
 }
 
 #[test]
 fn test_remove_from_inline_table() {
-    given(r#"
+    given(
+        r#"
         a = {a=2,  c = 3, b = 42}
-        b = {'hello' = "world"}"#
-    ).running(|root| {
+        b = {'hello' = "world"}"#,
+    )
+    .running(|root| {
         {
             let a = root.get_mut("a").unwrap();
             let a = as_inline_table!(a);
@@ -621,23 +688,27 @@ fn test_remove_from_inline_table() {
         assert_eq!(b.len(), 1);
         assert!(b.remove("hello").is_some());
         assert!(b.is_empty());
-    }).produces_display(r#"
+    })
+    .produces_display(
+        r#"
         a = {a=2, b = 42}
         b = {}
-"#
+"#,
     );
 }
 
 #[test]
 fn test_as_table_like() {
-    given(r#"
+    given(
+        r#"
         a = {a=2,  c = 3, b = 42}
         x = {}
         [[bin]]
         [b]
         x = "y"
-        [empty]"#
-    ).running(|root| {
+        [empty]"#,
+    )
+    .running(|root| {
         let a = root["a"].as_table_like();
         assert!(a.is_some());
         let a = a.unwrap();
@@ -653,7 +724,10 @@ fn test_as_table_like() {
         assert_eq!(b.get("x").and_then(Item::as_str), Some("y"));
 
         assert_eq!(root["x"].as_table_like().map(|t| t.iter().count()), Some(0));
-        assert_eq!(root["empty"].as_table_like().map(|t| t.is_empty()), Some(true));
+        assert_eq!(
+            root["empty"].as_table_like().map(|t| t.is_empty()),
+            Some(true)
+        );
 
         assert!(root["bin"].as_table_like().is_none());
     });
@@ -681,4 +755,43 @@ fn test_inline_table_append() {
     assert_eq!(b.len(), 3);
 }
 
-} // mod tests
+#[test]
+fn test_insert_dotted_into_std_table() {
+    given("")
+        .running(|root| {
+            root["nixpkgs"] = table();
+
+            root["nixpkgs"]["src"] = table();
+            root["nixpkgs"]["src"]
+                .as_table_like_mut()
+                .unwrap()
+                .set_dotted(true);
+            root["nixpkgs"]["src"]["git"] = value("https://github.com/nixos/nixpkgs");
+        })
+        .produces_display(
+            r#"
+[nixpkgs]
+src.git = "https://github.com/nixos/nixpkgs"
+"#,
+        );
+}
+
+#[test]
+fn test_insert_dotted_into_implicit_table() {
+    given("")
+        .running(|root| {
+            root["nixpkgs"] = table();
+
+            root["nixpkgs"]["src"]["git"] = value("https://github.com/nixos/nixpkgs");
+            root["nixpkgs"]["src"]
+                .as_table_like_mut()
+                .unwrap()
+                .set_dotted(true);
+        })
+        .produces_display(
+            r#"
+[nixpkgs]
+src.git = "https://github.com/nixos/nixpkgs"
+"#,
+        );
+}
